%% Preference-based Optimization
clc,clear
tic

% select "my_UQLab_folder/core" as your working directory; see README.md
uqlab
rng default
% variable
s0=[12	4 1	117	966	712];
% range of definition
lb=[10 0 0 50 50 50]';
ub=[14 15 120 1000 1000 1000]';
gaoptions = optimoptions('ga','ConstraintTolerance',1e-6,'PlotFcn', {@gaplotdistance,@gaplotscorediversity});
multioptions = optimoptions('gamultiobj','PlotFcn',{@gaplotpareto,@gaplotscorediversity});

%--- switch optimization problems
Name='MinE'; 
switch Name
    case 'MinE'     
        [s_opt,fval]=ga(@min_Emax,6,[],[],[],[],lb,ub,[],1:6,gaoptions);
    case 'MinA'
        [s_opt,fval]=ga(@min_Amax,6,[],[],[],[],lb,ub,[],1:6,gaoptions);
    case 'MaxE'
        [s_opt,fval]=ga(@max_Emax,6,[],[],[],[],lb,ub,[],1:6,gaoptions);
    case 'MaxA'
        [s_opt,fval]=ga(@max_Amax,6,[],[],[],[],lb,ub,[],1:6,gaoptions);
    case 'MinMax'
        [s_opt,fval]=ga(@minmax,6,[],[],[],[],lb,ub,[],1:6,gaoptions);
end

toc
function distance=minmax(s)
load('myModel_Emax','myModel_Emax')
load('myModel_Amax','myModel_Amax')

%--- energy perspective
E = uq_evalModel(myModel_Emax,s); 
Pe = 100+(0-100)/(1.4-0.1)*(E-0.1); 

%--- comfort perspective
A = uq_evalModel(myModel_Amax,s);
Pa = 100+ (0-100)/(0.26-0.21)*(A-0.21);  

%--- cost perspective
c_spacing=3000; 
c_type=6000; 
c_ratio=1.5; 

costcurve=fit([0.6 0.7 0.8 0.9 1]',[1 0.85 0.74 0.67 0.6]','smoothingspline');
c1=costcurve(0.05*s(1))*c_spacing;

if s(2)==0
    c2=c_type;
else
    c2=c_ratio*s(2)*c_type/15+(15-s(2))*c_type/15;
end
C=c1+c2;
MaxC=1.14*c_spacing+c_ratio*c_type;
MinC=0.85*c_spacing+c_type;
Pc = 100 + (0-100)/(MaxC-MinC)*(C-MinC); 

fprintf('Pc is %d\n ',Pc)
fprintf('Pe is %d\n ',Pe) 
fprintf('Pa is %d\n ',Pa) 
fprintf('c is %d\n ',C)
fprintf('E is %d\n ',E)
fprintf('A is %d\n ',A)

all = [-(Pe-100); -(Pa-100); -(Pc-100)];
distance=max(all);
end

function MinE=min_Emax(s)
load('myModel_Emax','myModel_Emax')
MinE = uq_evalModel(myModel_Emax,s); 
end


function MinA=min_Amax(s)
load('myModel_Amax','myModel_Amax')
MinA = uq_evalModel(myModel_Amax,s); 
end

function MaxE=max_Emax(s)
load('myModel_Emax','myModel_Emax')
Y = uq_evalModel(myModel_Emax,s); 
MaxE = -Y;
end

function MaxA=max_Amax(s)
load('myModel_Amax','myModel_Amax')
Y = uq_evalModel(myModel_Amax,s); 
MaxA = -Y;
end